@@ -6,6 +6,7 @@ import datetime
 
 import scipy
 from scipy import optimize
+import warnings
 
 
 scipy_methods = ['BFGS', 'L-BFGS-B', 'Newton-CG', 'CG', 'Powell', 'Nelder-Mead']
@@ -72,14 +73,18 @@ class Discrete:
     def _fast_fit(self):
         return ValueError('Fast or auto fit is not allowed for this distribution, please chose another value')
 
-    def _fit(self, data, bounds=None, init_params=np.array([1]), verbose=False, method='auto'):
+    def _fit(self, data, bounds=None, init_params=np.array([1]), verbose=False, method='auto', args=()):
         if method.lower() in ['auto', 'fast']:
             res = self._fast_fit(data)
         elif method in scipy_methods:
+            if args == ():
+                nll_args = (data,)
+            else:
+                nll_args = sum(((data,), args), ())
             res = scipy.optimize.minimize(
                 fun=self.nll_fun,
                 x0=init_params,
-                args=(data,),
+                args=nll_args,
                 method=method,
                 bounds=bounds
             )
@@ -304,7 +309,11 @@ class CMPoisson(Discrete):
         if (self.lambda_ is not None) & (self.nu_ is not None):
             assert self.lambda_ >= 0, ValueError('Value for parameter lambda must be strictly greater to 0 (lambda_ > 0)')
             assert self.nu_ >= 0, ValueError('Value for parameter nu must be greater or equal to 0 (nu_ >= 0)')
+            warnings.filterwarnings("error")
             self._Z = self.Z()
+            warnings.resetwarnings()
+
+        self.nll_fun = lambda params, data, j: self.nloglike(params=params, data=data, j=j)
 
     def Z(self, j=None) -> float:
         """Compute the :math:`Z` factor, normalizing constant.
@@ -331,15 +340,30 @@ class CMPoisson(Discrete):
         j = j if j else self.j
         z_i = 0
         for i in range(j):
+            _dec = False
+            # We use a decimal placeholder variable in case we encounter overflow errors (number is too long).
+            # We don't use decimal as a default but only in case of error because it slows down computations,
+            # therefore, by default we use light and simple format and call Decimal when needed.
+            # For instances where we need Decimal, we will transform back the ratio of decimals to float as
+            # the value will be of reasonable size and not too long for a classical float.
+
+            # Compute the denominator
             try:
-                _denom = math.factorial(i)**self.nu_
-                _dec = False
+                _denom = math.factorial(i) ** self.nu_
             except OverflowError:
-                _denom = Decimal(math.factorial(i))**Decimal(self.nu_)
+                # If overflow error (i.e. output is too long), use Decimal format
+                _denom = Decimal(math.factorial(i)) ** Decimal(self.nu_)
+                _dec = True
+            # Compute the numerator
+            try:
+                _num = self.lambda_ ** i
+            except RuntimeWarning:
+                # If runtime warning it is similar to overflow error (i.e. output is too long), use Decimal format
+                _num = Decimal(self.lambda_) ** Decimal(i)
                 _dec = True
 
-            _num = self.lambda_**i
             if _dec:
+                _denom = Decimal(_denom)
                 _num = Decimal(_num)
             z_i += float(_num / _denom)
 
@@ -365,9 +389,9 @@ class CMPoisson(Discrete):
         # Compute $\lambda^{x}$
         _pow = [pow(self.lambda_, _x) for _x in x]
 
+        _dec = False
         try:
             _pow = [float(p) for p in _pow]
-            _dec = False
         except OverflowError:
             # Inf the integers to compute are too large, we use decimal format
             # Ref: https://stackoverflow.com/questions/16174399/overflowerror-long-int-too-large-to-convert-to-float-in-python
@@ -401,21 +425,53 @@ class CMPoisson(Discrete):
         :return: Negative log-likelihood
         :rtype: :obj:`float`
         """
-        # TODO: improve function so param j can be changed from calling fit() function
         lambda_ = params[0]
         nu_ = params[1]
         X = np.asarray(data)
 
-        z_i = []
+        z_i = 0
         for i in range(j):
-            z_i += [(lambda_**i) / (math.factorial(i)**nu_)]
-        log_Z = np.log(np.sum(z_i))
+            _dec = False
+            # We use a decimal placeholder variable in case we encounter overflow errors (number is too long).
+            # We don't use decimal as a default but only in case of error because it slows down computations,
+            # therefore, by default we use light and simple format and call Decimal when needed.
+            # For instances where we need Decimal, we will transform back the ratio of decimals to float as
+            # the value will be of reasonable size and not too long for a classical float.
+
+            # Compute denominator
+            try:
+                # Try normal formula
+                _denom = math.factorial(i) ** nu_
+            except OverflowError:
+                # If overflow error (i.e. output is too long), use Decimal format
+                _denom = Decimal(math.factorial(i)) ** Decimal(nu_)
+                _dec = True
+
+            # Compute numerator
+            try:
+                _num = lambda_ ** i
+            except RuntimeWarning:
+                # If runtime warning it is similar to overflow error (i.e. output is too long), use Decimal format
+                _num = Decimal(lambda_) ** Decimal(i)
+                _dec = True
+
+            # If decimal was used
+            if _dec:
+                # If decimal was used for one of denominator or denominator, we apply decimal to all (for format consistency)
+                _denom = Decimal(_denom)
+                _num = Decimal(_num)
+
+            # Then compute the ratio and transform to float (whether decimal was used or not)
+            z_i += float(_num / _denom)
+
+        # Compute log(Z)
+        log_Z = np.log(z_i)
 
         _log_fact = np.asarray([math.log(math.factorial(_x)) for _x in X])
         ll = (math.log(lambda_) * np.sum(X)) - (nu_ * np.sum(_log_fact)) - (len(X) * log_Z)
         return -ll
 
-    def fit(self, data, method='L-BFGS-B', init_params=np.array([1., 1.]), **kwargs) -> dict:
+    def fit(self, data, method='L-BFGS-B', init_params=np.array([1., 1.]), j=None) -> dict:
         """Estimates the parameters :math:`\\lambda` and :math:`\\nu` of the distribution from empirical data based on Maximum Likelihood Estimation.
 
         .. note::
@@ -433,12 +489,16 @@ class CMPoisson(Discrete):
         :return: Estimated parameters
         :rtype: obj:`dict`
         """
+        # We transform warnings as error (for overflow) to handle in try / except: https://stackoverflow.com/questions/5644836/
+        warnings.filterwarnings("error")
+        j = j if j else self.j
         bounds = [(self.eps, None), (self.eps, None)]
 
-        res = self._fit(data=data, bounds=bounds, method=method, init_params=init_params, **kwargs)
+        res = self._fit(data=data, bounds=bounds, method=method, init_params=init_params, args=(j,))
         self.lambda_ = res.x[0]
         self.nu_ = res.x[1]
         self._Z = self.Z()
+        warnings.resetwarnings()
 
         out = {'lambda_': self.lambda_, 'nu_': self.nu_}
 
